@@ -24,7 +24,7 @@ use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
 );
 
 use vars qw($VERSION $PKTSIZE);
-$VERSION = '1.174';
+$VERSION = '1.176';
 $PKTSIZE = $^O eq 'linux' ? 3_000 : 100;
 
 use Carp qw(croak);
@@ -83,14 +83,7 @@ sub spawn {
   croak "$type requires an even number of parameters" if @_ % 2;
   my %params = @_;
 
-  # If we aren't given a socket, then we'll need privileges to create
-  # one ourselves.
-
-  croak "$type requires root privilege" if (
-    $> and ($^O ne "VMS") and
-    ($^O ne "cygwin") and
-    not defined $params{Socket}
-  );
+  croak "$type requires root privilege" unless can_open_socket();
 
   my $alias         = delete $params{Alias};
   $alias            = "pinger" unless defined $alias and length $alias;
@@ -156,24 +149,29 @@ sub poco_ping_start {
 }
 
 
+# (NOT A POE EVENT HANDLER)
+# Test whether this process can open raw sockets.
+sub can_open_socket {
+  my $socket = eval { _create_handle() };
+  return 0 if $@ or not $socket;
+  return 1;
+}
+
+
 # (NOT A POE EVENT HANDLER)
 # Create a raw socket to send ICMP packets down.
 # (optionally) mess with the size of the buffers on the socket.
 
 sub _create_handle {
-  my ($kernel, $heap) = @_;
   DEBUG_SOCKET and warn "opening a raw socket for icmp";
 
-  my $protocol = (getprotobyname('icmp'))[2]
-    or die "can't get icmp protocol by name: $!";
+  my $protocol = Socket::IPPROTO_ICMP;
 
   my $socket = gensym();
   socket($socket, PF_INET, SOCK_RAW, $protocol)
     or die "can't create icmp socket: $!";
 
-  $heap->{socket_handle} = $socket;
-
-  _setup_handle($kernel, $heap);
+  return $socket;
 }
 
 ### NOT A POE EVENT HANDLER
@@ -225,26 +223,19 @@ sub poco_ping_ping {
 
   DEBUG and warn "ping requested for $address ($tries_left try/tries left)\n";
 
-  _do_ping(
-    $kernel, $heap, $sender, $event, $address, $timeout, $tries_left, 0
-  );
+  _do_ping($kernel, $heap, $sender, $event, $address, $timeout, $tries_left);
 }
 
 
 sub _do_ping {
-  my (
-    $kernel, $heap, $sender, $event, $address, $timeout, $tries_left,
-    $is_a_retry
-  ) = @_;
+  my ($kernel, $heap, $sender, $event, $address, $timeout, $tries_left) = @_;
 
   # No current pings.  Open a socket, or setup the existing one.
   unless (scalar(keys %{$heap->{ping_by_seq}})) {
     unless (defined $heap->{socket_handle}) {
-      _create_handle($kernel, $heap);
-    }
-    else {
-      _setup_handle($kernel, $heap);
+      $heap->{socket_handle} = _create_handle();
     }
+    _setup_handle($kernel, $heap);
   }
 
   # Get the timeout, or default to the one set for the component.
@@ -342,7 +333,6 @@ sub _do_ping {
     $event,             # PEND_EVENT
     $address,           # PEND_ADDR ???
     $timeout,           # PEND_TIMEOUT
-    $is_a_retry,        # PEND_IS_RETRY
   ];
 
   if ($tries_left and $tries_left > 1) {
@@ -386,7 +376,6 @@ sub _send_next_packet {
     $event,             # PEND_EVENT
     $address,           # PEND_ADDR ???
     $timeout,           # PEND_TIMEOUT
-    $is_a_retry,        # PEND_IS_RETRY
   ) = @$ping_info;
 
   # Send the packet.  If send() fails, then we bail with an error.
@@ -421,26 +410,19 @@ sub _send_next_packet {
   $kernel->delay( $seq => $timeout );
 
   DEBUG_PBS and warn "recording ping_by_seq($seq)";
-  if ($is_a_retry) {
-    # If retries, set the request time to the new/actual request time.
-    # Inserted by Ralph Schmitt 2009-09-12.
-    $heap->{ping_by_seq}->{$seq}->[PBS_REQUEST_TIME] = time();
-  }
-  else {
-    $heap->{ping_by_seq}->{$seq} = [
-      # PBS_POSTBACK
-      $sender->postback(
-        $event,
-        $address,    # REQ_ADDRESS
-        $timeout,    # REQ_TIMEOUT
-        time(),      # REQ_TIME
-        @user_args,  # REQ_USER_ARGS
-      ),
-      "$sender",   # PBS_SESSION (stringified to weaken reference)
-      $address,    # PBS_ADDRESS
-      time()       # PBS_REQUEST_TIME
-    ];
-  }
+  $heap->{ping_by_seq}->{$seq} = [
+    # PBS_POSTBACK
+    $sender->postback(
+      $event,
+      $address,    # REQ_ADDRESS
+      $timeout,    # REQ_TIMEOUT
+      time(),      # REQ_TIME
+      @user_args,  # REQ_USER_ARGS
+    ),
+    "$sender",   # PBS_SESSION (stringified to weaken reference)
+    $address,    # PBS_ADDRESS
+    time()       # PBS_REQUEST_TIME
+  ];
 
   # Duplicate pings?  Forcibly time out the previous one.
   if (exists $heap->{addr_to_seq}->{$sender}->{$address}) {
@@ -616,7 +598,7 @@ sub poco_ping_pong {
     warn ",----- packet from ", inet_ntoa($from_ip), ", port $from_port\n";
     warn "| type = $from_type / subtype = $from_subcode\n";
     warn "| checksum = $from_checksum, pid = $from_pid, seq = $from_seq\n";
-    warn "| message: $from_message\n";
+    warn "| message: ", unpack("H*", $from_message), "\n";
     warn "`------------------------------------------------------------\n";
   };
 
@@ -674,7 +656,10 @@ sub poco_ping_default {
   if ($retryinfo) {
     my ($sender, $event, $address, $timeout, $remaining) = @{$retryinfo};
     DEBUG and warn("retrying ping for $address (", $remaining - 1, ")\n");
-    _do_ping($kernel, $heap, $sender, $event, $address, $remaining - 1, 1);
+    _do_ping(
+      $kernel, $heap, $sender, $event, $address, $timeout,
+      $remaining - 1
+    );
     return;
   }
 
@@ -1035,7 +1020,7 @@ http://search.cpan.org/dist/POE-Component-Client-Ping/
 
 =head1 AUTHOR & COPYRIGHTS
 
-POE::Component::Client::Ping is Copyright 1999-2009 by Rocco Caputo.
+POE::Component::Client::Ping is Copyright 1999-2020 by Rocco Caputo.
 All rights are reserved.  POE::Component::Client::Ping is free
 software; you may redistribute it and/or modify it under the same
 terms as Perl itself.
